@@ -19,6 +19,7 @@
 #include <glib-object.h>
 #include "languages.h"
 #include "keyboard.h"
+#include "system-layouts.h"
 
 enum
 {
@@ -373,6 +374,7 @@ static void onSourcesChanged (GSettings *pSettings, const gchar *sKey, gpointer
 static void keyboard_init(Keyboard *self)
 {
     self->pPrivate = keyboard_get_instance_private(self);
+    self->pPrivate->lLayoutRec = NULL;
     self->pPrivate->lLayouts = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, freeLayout);
 
     // Read all available layouts
@@ -445,111 +447,22 @@ static void keyboard_init(Keyboard *self)
     }
     else
     {
-        // Get layouts from /etc/default/keyboard
-        gchar *sFile;
-        g_file_get_contents ("/etc/default/keyboard", &sFile, NULL, NULL);
-        gchar **lLines = g_strsplit(sFile, "\n", -1);
-        guint nLines = g_strv_length(lLines);
-        gchar **lLayouts = NULL;
-        gchar **lVariants = NULL;
+        // Get system layouts
+        getSystemLayouts ("/etc/default/keyboard", &self->pPrivate->lLayoutRec, NULL, FALSE);
 
-        for (guint nLine = 0; nLine < nLines; nLine++)
+        if (!self->pPrivate->lLayoutRec)
         {
-            gboolean bIsLayout = g_str_has_prefix(lLines[nLine], "XKBLAYOUT");
-
-            if (bIsLayout == TRUE)
-            {
-                gboolean bQuoted = g_strrstr(lLines[nLine], "\"") != NULL;
-                gchar *sLayout = NULL;
-
-                if (bQuoted == TRUE)
-                {
-                    sLayout = (lLines[nLine] + 11);
-                    guint nLength = strlen(sLayout);
-                    sLayout[nLength - 1] = '\0';
-                }
-                else
-                {
-                    sLayout = (lLines[nLine] + 10);
-                }
-
-                lLayouts = g_strsplit(sLayout, ",", -1);
-
-                continue;
-            }
-
-            gboolean bIsVariant = g_str_has_prefix(lLines[nLine], "XKBVARIANT");
-
-            if (bIsVariant == TRUE)
-            {
-                gboolean bQuoted = g_strrstr(lLines[nLine], "\"") != NULL;
-                gchar *sVariant = NULL;
-
-                if (bQuoted == TRUE)
-                {
-                    sVariant = (lLines[nLine] + 12);
-                    guint nLength = strlen(sVariant);
-                    sVariant[nLength - 1] = '\0';
-                }
-                else
-                {
-                    sVariant = (lLines[nLine] + 11);
-                }
-
-                lVariants = g_strsplit(sVariant, ",", -1);
-
-                continue;
-            }
+            getSystemLayouts ("/etc/X11/xorg.conf.d/00-keyboard.conf", &self->pPrivate->lLayoutRec, NULL, FALSE);
         }
 
-        guint nVariants = 0;
-
-        if (lVariants != NULL)
+        if (!self->pPrivate->lLayoutRec)
         {
-            g_strv_length(lVariants);
+            gchar *sId = g_strdup ("us");
+            self->pPrivate->lLayoutRec = g_slist_append (self->pPrivate->lLayoutRec, sId);
         }
 
-        if (lLayouts != NULL)
-        {
-            guint nLayouts = g_strv_length(lLayouts);
-
-            for (guint nLayout = 0; nLayout < nLayouts; nLayout++)
-            {
-                gchar *sId = NULL;
-
-                if (nVariants > nLayout)
-                {
-                    guint nVariant = strlen(lVariants[nLayout]);
-
-                    if (nVariants == nLayouts && nVariant > 0)
-                    {
-                        sId = g_strconcat(lLayouts[nLayout], "+", lVariants[nLayout], NULL);
-                    }
-                    else
-                    {
-                        sId = g_strdup(lLayouts[nLayout]);
-                    }
-                }
-                else
-                {
-                    sId = g_strdup(lLayouts[nLayout]);
-                }
-
-                self->pPrivate->lLayoutRec = g_slist_append(self->pPrivate->lLayoutRec, sId);
-            }
-
-            self->pPrivate->nLayout = 0;
-
-            g_strfreev(lLayouts);
-        }
-
-        if (lVariants != NULL)
-        {
-            g_strfreev(lVariants);
-        }
-
-        g_strfreev(lLines);
-        g_free(sFile);
+        self->pPrivate->nLayout = 0;
+        //~ Get system layouts
 
         ActUserManager *pManager = act_user_manager_get_default();
         gboolean bIsLoaded;
