@@ -1,5 +1,5 @@
 /*
- * Copyright 2021-2023 Robert Tari <robert@tari.in>
+ * Copyright 2021-2024 Robert Tari <robert@tari.in>
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 3, as published
@@ -20,6 +20,7 @@
 #include <libxklavier/xklavier.h>
 #include "languages.h"
 #include "keyboard.h"
+#include "system-layouts.h"
 
 #define GREETER_BUS_NAME "org.ayatana.greeter"
 #define GREETER_BUS_PATH "/org/ayatana/greeter"
@@ -48,7 +49,7 @@ struct _KeyboardPrivate
     GDBusConnection *pConnection;
     guint nSubscription;
     gchar *sUser;
-    gchar *sSystemLayout;
+    GStrv lSystemLayouts;
 };
 
 typedef KeyboardPrivate priv_t;
@@ -148,7 +149,13 @@ static void getAccountsService(Keyboard *pKeyboard, ActUser *pUser)
             g_slist_free_full (g_steal_pointer (&pKeyboard->pPrivate->lLayoutRec), g_free);
         }
 
-        pKeyboard->pPrivate->lLayoutRec = g_slist_append (pKeyboard->pPrivate->lLayoutRec, g_strdup (pKeyboard->pPrivate->sSystemLayout));
+        guint nLayouts = g_strv_length (pKeyboard->pPrivate->lSystemLayouts);
+
+        for (guint nLayout = 0; nLayout < nLayouts; nLayout++)
+        {
+            pKeyboard->pPrivate->lLayoutRec = g_slist_append (pKeyboard->pPrivate->lLayoutRec, g_strdup (pKeyboard->pPrivate->lSystemLayouts[nLayout]));
+        }
+
         g_timeout_add(500, (GSourceFunc)emitDelayedSignal, pKeyboard);
     }
     else
@@ -187,7 +194,12 @@ static void getAccountsService(Keyboard *pKeyboard, ActUser *pUser)
 
             if (!nLayouts)
             {
-                pKeyboard->pPrivate->lLayoutRec = g_slist_append (pKeyboard->pPrivate->lLayoutRec, g_strdup (pKeyboard->pPrivate->sSystemLayout));
+                guint nLayouts = g_strv_length (pKeyboard->pPrivate->lSystemLayouts);
+
+                for (guint nLayout = 0; nLayout < nLayouts; nLayout++)
+                {
+                    pKeyboard->pPrivate->lLayoutRec = g_slist_append (pKeyboard->pPrivate->lLayoutRec, g_strdup (pKeyboard->pPrivate->lSystemLayouts[nLayout]));
+                }
             }
 
             g_variant_iter_free(pIter);
@@ -494,9 +506,9 @@ static void onDispose(GObject *pObject)
         g_slist_free_full(self->pPrivate->lLayoutRec, g_free);
     }
 
-    if (self->pPrivate->sSystemLayout)
+    if (self->pPrivate->lSystemLayouts)
     {
-        g_free (self->pPrivate->sSystemLayout);
+        g_strfreev (self->pPrivate->lSystemLayouts);
     }
 
     G_OBJECT_CLASS(keyboard_parent_class)->dispose(pObject);
@@ -556,6 +568,7 @@ static void onUserChanged (GDBusConnection *pConnection, const gchar *sSender, c
 static void keyboard_init(Keyboard *self)
 {
     self->pPrivate = keyboard_get_instance_private(self);
+    self->pPrivate->lLayoutRec = NULL;
     self->pPrivate->lLayouts = g_hash_table_new_full(g_str_hash, g_str_equal, NULL, freeLayout);
 
     // Read all available layouts
@@ -645,145 +658,26 @@ static void keyboard_init(Keyboard *self)
         self->pPrivate->lUsers = NULL;
         self->pPrivate->nSubscription = g_dbus_connection_signal_subscribe (self->pPrivate->pConnection, NULL, GREETER_BUS_NAME, "UserChanged", GREETER_BUS_PATH, NULL, G_DBUS_SIGNAL_FLAGS_NONE, onUserChanged, self, NULL);
 
-        // Get system layout
-        gboolean bDefaultLocation = g_file_test ("/etc/default/keyboard", G_FILE_TEST_EXISTS);
-        gchar *sLocation = NULL;
+        // Get system layouts
+        getSystemLayouts ("/etc/default/keyboard", &self->pPrivate->lLayoutRec, &self->pPrivate->lSystemLayouts, TRUE);
 
-        if (bDefaultLocation)
+        if (!self->pPrivate->lLayoutRec)
         {
-            sLocation = "/etc/default/keyboard";
-        }
-        else
-        {
-            sLocation = "/etc/X11/xorg.conf.d/00-keyboard.conf";
+            getSystemLayouts ("/etc/X11/xorg.conf.d/00-keyboard.conf", &self->pPrivate->lLayoutRec, &self->pPrivate->lSystemLayouts, TRUE);
         }
 
-        gchar *sFile = NULL;
-        GError *pError = NULL;
-        g_file_get_contents (sLocation, &sFile, NULL, &pError);
-        gchar *sLayout = NULL;
-        gchar *sVariant = NULL;
-
-        if (!pError)
+        if (!self->pPrivate->lLayoutRec)
         {
-            GRegex *pRegex = NULL;
-
-            if (bDefaultLocation)
-            {
-                #if GLIB_CHECK_VERSION(2, 73, 0)
-                    pRegex = g_regex_new (" *XKBLAYOUT *= *\"*([,a-zA-Z0-9]*)\"*", G_REGEX_DEFAULT, G_REGEX_MATCH_DEFAULT, &pError);
-                #else
-                    pRegex = g_regex_new (" *XKBLAYOUT *= *\"*([,a-zA-Z0-9]*)\"*", (GRegexCompileFlags) 0, (GRegexMatchFlags) 0, &pError);
-                #endif
-            }
-            else
-            {
-                #if GLIB_CHECK_VERSION(2, 73, 0)
-                    pRegex = g_regex_new (" *Option +\"*XkbLayout\"* +\"*([,a-zA-Z0-9]*)\"*", G_REGEX_DEFAULT, G_REGEX_MATCH_DEFAULT, &pError);
-                #else
-                    pRegex = g_regex_new (" *Option +\"*XkbLayout\"* +\"*([,a-zA-Z0-9]*)\"*", (GRegexCompileFlags) 0, (GRegexMatchFlags) 0, &pError);
-                #endif
-            }
-
-            if (!pError)
-            {
-                GMatchInfo *pMatchInfo = NULL;
-
-                #if GLIB_CHECK_VERSION(2, 73, 0)
-                    gboolean bMatch = g_regex_match (pRegex, sFile, G_REGEX_MATCH_DEFAULT, &pMatchInfo);
-                #else
-                    gboolean bMatch = g_regex_match (pRegex, sFile, (GRegexMatchFlags) 0, &pMatchInfo);
-                #endif
-
-                if (bMatch)
-                {
-                    sLayout = g_match_info_fetch (pMatchInfo, 1);
-                }
-                else
-                {
-                    g_error ("PANIC: No system XkbLayout found");
-                }
-
-                g_match_info_free (pMatchInfo);
-                g_regex_unref (pRegex);
-            }
-            else
-            {
-                g_error ("PANIC: Failed to compile regex: %s", pError->message);
-                g_error_free (pError);
-            }
-
-            if (bDefaultLocation)
-            {
-                #if GLIB_CHECK_VERSION(2, 73, 0)
-                    pRegex = g_regex_new (" *XKBVARIANT *= *\"*([,a-zA-Z0-9]*)\"*", G_REGEX_DEFAULT, G_REGEX_MATCH_DEFAULT, &pError);
-                #else
-                    pRegex = g_regex_new (" *XKBVARIANT *= *\"*([,a-zA-Z0-9]*)\"*", (GRegexCompileFlags) 0, (GRegexMatchFlags) 0, &pError);
-                #endif
-            }
-            else
-            {
-                #if GLIB_CHECK_VERSION(2, 73, 0)
-                    pRegex = g_regex_new (" *Option +\"*XkbVariant\"* +\"*([,a-zA-Z0-9]*)\"*", G_REGEX_DEFAULT, G_REGEX_MATCH_DEFAULT, &pError);
-                #else
-                    pRegex = g_regex_new (" *Option +\"*XkbVariant\"* +\"*([,a-zA-Z0-9]*)\"*", (GRegexCompileFlags) 0, (GRegexMatchFlags) 0, &pError);
-                #endif
-            }
-
-            if (!pError)
-            {
-                GMatchInfo *pMatchInfo = NULL;
-
-                #if GLIB_CHECK_VERSION(2, 73, 0)
-                    gboolean bMatch = g_regex_match (pRegex, sFile, G_REGEX_MATCH_DEFAULT, &pMatchInfo);
-                #else
-                    gboolean bMatch = g_regex_match (pRegex, sFile, (GRegexMatchFlags) 0, &pMatchInfo);
-                #endif
-
-                if (bMatch)
-                {
-                    sVariant = g_match_info_fetch (pMatchInfo, 1);
-                }
-
-                g_match_info_free (pMatchInfo);
-                g_regex_unref (pRegex);
-            }
-            else
-            {
-                g_error ("PANIC: Failed to compile regex: %s", pError->message);
-                g_error_free (pError);
-            }
-
-            g_free(sFile);
-        }
-        else
-        {
-            g_error ("PANIC: Failed to get %s contents: %s", sLocation, pError->message);
-            g_error_free (pError);
+            GStrvBuilder *pBuilder = g_strv_builder_new ();
+            gchar *sId = g_strdup ("us");
+            self->pPrivate->lLayoutRec = g_slist_append (self->pPrivate->lLayoutRec, sId);
+            g_strv_builder_add (pBuilder, sId);
+            self->pPrivate->lSystemLayouts = g_strv_builder_end (pBuilder);
+            g_strv_builder_unref (pBuilder);
         }
 
-        gchar *sId = NULL;
-        guint nVariant = 0;
-
-        if (sVariant)
-        {
-            nVariant = strlen (sVariant);
-        }
-
-        if (nVariant)
-        {
-            sId = g_strconcat (sLayout, "+", sVariant, NULL);
-        }
-        else
-        {
-            sId = g_strdup (sLayout);
-        }
-
-        g_free (sLayout);
-        g_free (sVariant);
-        self->pPrivate->lLayoutRec = g_slist_append (self->pPrivate->lLayoutRec, sId);
-        self->pPrivate->sSystemLayout = g_strdup (sId);
         self->pPrivate->nLayout = 0;
+        //~ Get system layouts
 
         ActUserManager *pManager = act_user_manager_get_default();
         gboolean bIsLoaded;
